@@ -17,52 +17,31 @@
 #ifndef PYI_LAUNCH_H
 #define PYI_LAUNCH_H
 
-#include "pyi_archive.h"
-#include "pyi_splash.h"
-
-/*****************************************************************
-* The following 4 entries are for applications which may need to
-* use to 2 steps to execute
-*****************************************************************/
+typedef struct _pyi_context PYI_CONTEXT;
 
 /*
- * Extract binaries from the archive
- *
- * @param archive_status     The archive from which the binaries will
- *                           be extracted.
- *
- * @param splash_status      The splash screen status which describes
- *                           on which progress updated to occur. May
- *                           be NULL
- *
- * @return 0 on success, non-zero otherwise.
+ * Extract files from embedded archive (onefile mode).
  */
-int pyi_launch_extract_binaries(ARCHIVE_STATUS *archive_status,
-                                SPLASH_STATUS *splash_status);
+int pyi_launch_extract_files_from_archive(PYI_CONTEXT *pyi_ctx);
 
 /*
  * Wrapped platform specific initialization before loading Python and executing
  * all scripts in the archive.
  */
-void pyi_launch_initialize(ARCHIVE_STATUS *archive_status);
+void pyi_launch_initialize(PYI_CONTEXT *pyi_ctx);
 
 /*
  * Wrapped platform specific finalization before loading Python and executing
  * all scripts in the archive.
  */
-void pyi_launch_finalize(ARCHIVE_STATUS *archive_status);
+void pyi_launch_finalize(PYI_CONTEXT *pyi_ctx);
 
 /*
  * Load Python and execute all scripts in the archive
  *
  * @return -1 for internal failures, or the rc of the last script.
  */
-int pyi_launch_execute(ARCHIVE_STATUS *status);
+int pyi_launch_execute(PYI_CONTEXT *pyi_ctx);
 
-/*
- * Transform parent process to background (OSX only).
- */
-void pyi_parent_to_background();
-
-#endif  /* PYI_LAUNCH_H */
 
+#endif /* PYI_LAUNCH_H */
