@@ -19,9 +19,10 @@
 #include <string.h>
 
 #include "pyi_pyconfig.h"
+#include "pyi_archive.h"
 #include "pyi_global.h"
+#include "pyi_main.h"
 #include "pyi_utils.h"
-#include "pyi_win32_utils.h"
 
 
 /*
@@ -63,11 +64,11 @@ pyi_runtime_options_free(PyiRuntimeOptions *options)
 static int
 _pyi_copy_xwflag(const char *flag, wchar_t **pdest_buf)
 {
-    wchar_t flag_w[PATH_MAX + 1];
+    wchar_t flag_w[PYI_PATH_MAX + 1];
 
     /* Convert multi-byte string to wide-char. The multibyte encoding in PKG is UTF-8,
      * but W and X options should consist only of ASCII characters. */
-    if (mbstowcs(flag_w, flag, PATH_MAX) < 0) {
+    if (mbstowcs(flag_w, flag, PYI_PATH_MAX) < 0) {
         return -1;
     }
 
@@ -137,10 +138,11 @@ _pyi_match_and_parse_xflag(const char *flag, const char *name, int *dest_var)
  * options found in the PKG archive.
  */
 PyiRuntimeOptions *
-pyi_runtime_options_read(const ARCHIVE_STATUS *archive_status)
+pyi_runtime_options_read(const PYI_CONTEXT *pyi_ctx)
 {
     PyiRuntimeOptions *options;
-    const TOC *ptoc;
+    const ARCHIVE *archive = pyi_ctx->archive;
+    const TOC_ENTRY *toc_entry;
     int num_wflags = 0;
     int num_xflags = 0;
     int failed = 0;
@@ -154,46 +156,51 @@ pyi_runtime_options_read(const ARCHIVE_STATUS *archive_status)
     options->utf8_mode = -1; /* default: auto-select based on locale */
 
     /* Parse run-time options from PKG archive */
-    for (ptoc = archive_status->tocbuff; ptoc < archive_status->tocend; ptoc = pyi_arch_increment_toc_ptr(archive_status, ptoc)) {
+    for (toc_entry = archive->toc; toc_entry < archive->toc_end; toc_entry = pyi_archive_next_toc_entry(archive, toc_entry)) {
         const char *value_str;
 
+        /* We are only interested in OPTION entries */
+        if (toc_entry->typecode != ARCHIVE_ITEM_RUNTIME_OPTION) {
+            continue;
+        }
+
         /* Skip bootloader options; these start with "pyi-" */
-        if (strncmp(ptoc->name, "pyi-", 4) == 0) {
+        if (strncmp(toc_entry->name, "pyi-", 4) == 0) {
             continue;
         }
 
         /* Verbose flag: v, verbose */
-        if (strcmp(ptoc->name, "v") == 0 || strcmp(ptoc->name, "verbose") == 0) {
+        if (strcmp(toc_entry->name, "v") == 0 || strcmp(toc_entry->name, "verbose") == 0) {
             options->verbose++;
             continue;
         }
 
         /* Unbuffered flag: u, unbuffered */
-        if (strcmp(ptoc->name, "u") == 0 || strcmp(ptoc->name, "unbuffered") == 0) {
+        if (strcmp(toc_entry->name, "u") == 0 || strcmp(toc_entry->name, "unbuffered") == 0) {
             options->unbuffered = 1;
             continue;
         }
 
         /* Optimize flag: O, optimize */
-        if (strcmp(ptoc->name, "O") == 0 || strcmp(ptoc->name, "optimize") == 0) {
+        if (strcmp(toc_entry->name, "O") == 0 || strcmp(toc_entry->name, "optimize") == 0) {
             options->optimize++;
             continue;
         }
 
         /* W flag: W <warning_rule> */
-        if (strncmp(ptoc->name, "W ", 2) == 0) {
+        if (strncmp(toc_entry->name, "W ", 2) == 0) {
             num_wflags++;
             continue;
         }
 
         /* X flag: X <key=value> */
-        if (strncmp(ptoc->name, "X ", 2) == 0) {
+        if (strncmp(toc_entry->name, "X ", 2) == 0) {
             num_xflags++;
             continue;
         }
 
         /* Hash seed flag: hash_seed=value */
-        value_str = _pyi_match_key_value_flag(ptoc->name, "hash_seed");
+        value_str = _pyi_match_key_value_flag(toc_entry->name, "hash_seed");
         if (value_str && value_str[0]) {
             options->use_hash_seed = 1;
             options->hash_seed = strtoul(value_str, NULL, 10);
@@ -212,18 +219,23 @@ pyi_runtime_options_read(const ARCHIVE_STATUS *archive_status)
     }
 
     /* Collect */
-    for (ptoc = archive_status->tocbuff; ptoc < archive_status->tocend; ptoc = pyi_arch_increment_toc_ptr(archive_status, ptoc)) {
-        if (strncmp(ptoc->name, "W ", 2) == 0) {
+    for (toc_entry = archive->toc; toc_entry < archive->toc_end; toc_entry = pyi_archive_next_toc_entry(archive, toc_entry)) {
+        /* We are only interested in OPTION entries */
+        if (toc_entry->typecode != ARCHIVE_ITEM_RUNTIME_OPTION) {
+            continue;
+        }
+
+        if (strncmp(toc_entry->name, "W ", 2) == 0) {
             /* Copy for pass-through */
-            const char *flag = &ptoc->name[2]; /* Skip first two characters */
+            const char *flag = &toc_entry->name[2]; /* Skip first two characters */
             if (_pyi_copy_xwflag(flag, &options->wflags[options->num_wflags]) < 0) {
                 failed = 1;
                 goto end;
             }
             options->num_wflags++;
-        } else if (strncmp(ptoc->name, "X ", 2) == 0) {
+        } else if (strncmp(toc_entry->name, "X ", 2) == 0) {
             /* Copy for pass-through */
-            const char *flag = &ptoc->name[2]; /* Skip first two characters */
+            const char *flag = &toc_entry->name[2]; /* Skip first two characters */
             if (_pyi_copy_xwflag(flag, &options->xflags[options->num_xflags]) < 0) {
                 failed = 1;
                 goto end;
@@ -233,8 +245,6 @@ pyi_runtime_options_read(const ARCHIVE_STATUS *archive_status)
             /* Try matching the utf8 and dev X-flag */
             _pyi_match_and_parse_xflag(flag, "utf8", &options->utf8_mode);
             _pyi_match_and_parse_xflag(flag, "dev", &options->dev_mode);
-        } else {
-            continue;
         }
     }
 
@@ -262,7 +272,7 @@ _pyi_pyconfig_set_string(PyConfig *config, wchar_t **dest_field, const char *str
 
 #ifdef _WIN32
     wchar_t *str_w;
-    str_w = pyi_win32_utils_from_utf8(NULL, str, 0);
+    str_w = pyi_win32_utf8_to_wcs(str, NULL, 0);
     if (!str_w) {
         return -1;
     }
@@ -280,7 +290,7 @@ _pyi_pyconfig_set_string(PyConfig *config, wchar_t **dest_field, const char *str
  * Allocate the PyConfig structure, based on the python version.
  */
 PyConfig *
-pyi_pyconfig_create()
+pyi_pyconfig_create(int python_version)
 {
     /* Macro to avoid manual code repetition. */
     #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
@@ -289,7 +299,7 @@ pyi_pyconfig_create()
     }
     /* Macro end */
 
-    switch (pyvers) {
+    switch (python_version) {
         _IMPL_CASE(308, PyConfig_v38)
         _IMPL_CASE(309, PyConfig_v39)
         _IMPL_CASE(310, PyConfig_v310)
@@ -323,20 +333,20 @@ pyi_pyconfig_free(PyConfig *config)
  * Set program name. Used to set sys.executable, and in early error messages.
  */
 int
-pyi_pyconfig_set_program_name(PyConfig *config, const ARCHIVE_STATUS *archive_status)
+pyi_pyconfig_set_program_name(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
 {
     /* Macro to avoid manual code repetition. */
     #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
     case PY_VERSION: { \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
-        if (_pyi_pyconfig_set_string(config, &config_impl->program_name, archive_status->executablename) < 0) { \
+        if (_pyi_pyconfig_set_string(config, &config_impl->program_name, pyi_ctx->executable_filename) < 0) { \
             return -1; \
         } \
         return 0; \
     }
     /* Macro end */
 
-    switch (pyvers) {
+    switch (pyi_ctx->archive->python_version) {
         _IMPL_CASE(308, PyConfig_v38)
         _IMPL_CASE(309, PyConfig_v39)
         _IMPL_CASE(310, PyConfig_v310)
@@ -356,17 +366,17 @@ pyi_pyconfig_set_program_name(PyConfig *config, const ARCHIVE_STATUS *archive_st
  * Set python home directory. Used to set sys.prefix.
  */
 int
-pyi_pyconfig_set_python_home(PyConfig *config, const ARCHIVE_STATUS *archive_status)
+pyi_pyconfig_set_python_home(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
 {
     /* Macro to avoid manual code repetition. */
     #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
     case PY_VERSION: { \
         PYCONFIG_IMPL *config_impl = (PYCONFIG_IMPL *)config; \
-        return _pyi_pyconfig_set_string(config, &config_impl->home, archive_status->mainpath); \
+        return _pyi_pyconfig_set_string(config, &config_impl->home, pyi_ctx->application_home_dir); \
     }
     /* Macro end */
 
-    switch (pyvers) {
+    switch (pyi_ctx->archive->python_version) {
         _IMPL_CASE(308, PyConfig_v38)
         _IMPL_CASE(309, PyConfig_v39)
         _IMPL_CASE(310, PyConfig_v310)
@@ -392,7 +402,7 @@ pyi_pyconfig_set_python_home(PyConfig *config, const ARCHIVE_STATUS *archive_sta
  * sys.path.
  */
 static int
-_pyi_pyconfig_set_module_search_paths(PyConfig *config, int num_paths, wchar_t **paths)
+_pyi_pyconfig_set_module_search_paths(PyConfig *config, int python_version, int num_paths, wchar_t **paths)
 {
     /* Macro to avoid manual code repetition. */
     #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
@@ -405,7 +415,7 @@ _pyi_pyconfig_set_module_search_paths(PyConfig *config, int num_paths, wchar_t *
     }
     /* Macro end */
 
-    switch (pyvers) {
+    switch (python_version) {
         _IMPL_CASE(308, PyConfig_v38)
         _IMPL_CASE(309, PyConfig_v39)
         _IMPL_CASE(310, PyConfig_v310)
@@ -422,13 +432,13 @@ _pyi_pyconfig_set_module_search_paths(PyConfig *config, int num_paths, wchar_t *
 }
 
 int
-pyi_pyconfig_set_module_search_paths(PyConfig *config, const ARCHIVE_STATUS *archive_status)
+pyi_pyconfig_set_module_search_paths(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
 {
     /* TODO: instead of stitching together char strings and converting
      * them, we could probably stitch together wide-char strings directly,
      * as `home` field in config structure has already been converted. */
-    char base_library_path[PATH_MAX + 1];
-    char lib_dynload_path[PATH_MAX + 1];
+    char base_library_path[PYI_PATH_MAX + 1];
+    char lib_dynload_path[PYI_PATH_MAX + 1];
 
     const char *module_search_paths[3];
     wchar_t *module_search_paths_w[3];
@@ -437,23 +447,23 @@ pyi_pyconfig_set_module_search_paths(PyConfig *config, const ARCHIVE_STATUS *arc
     int i;
 
     /* home/base_library.zip */
-    if (snprintf(base_library_path, PATH_MAX, "%s%c%s", archive_status->mainpath, PYI_SEP, "base_library.zip") >= PATH_MAX) {
+    if (snprintf(base_library_path, PYI_PATH_MAX, "%s%c%s", pyi_ctx->application_home_dir, PYI_SEP, "base_library.zip") >= PYI_PATH_MAX) {
         return -1;
     }
 
     /* home/lib-dynload */
-    if (snprintf(lib_dynload_path, PATH_MAX, "%s%c%s", archive_status->mainpath, PYI_SEP, "lib-dynload") >= PATH_MAX) {
+    if (snprintf(lib_dynload_path, PYI_PATH_MAX, "%s%c%s", pyi_ctx->application_home_dir, PYI_SEP, "lib-dynload") >= PYI_PATH_MAX) {
         return -1;
     }
 
     module_search_paths[0] = base_library_path;
     module_search_paths[1] = lib_dynload_path;
-    module_search_paths[2] = archive_status->mainpath;
+    module_search_paths[2] = pyi_ctx->application_home_dir;
 
     /* Convert */
     for (i = 0; i < 3; i++) {
 #ifdef _WIN32
-        module_search_paths_w[i] = pyi_win32_utils_from_utf8(NULL, module_search_paths[i], 0);
+        module_search_paths_w[i] = pyi_win32_utf8_to_wcs(module_search_paths[i], NULL, 0);
 #else
         module_search_paths_w[i] = PI_Py_DecodeLocale(module_search_paths[i], NULL);
 #endif
@@ -467,7 +477,12 @@ pyi_pyconfig_set_module_search_paths(PyConfig *config, const ARCHIVE_STATUS *arc
     }
 
     /* Set */
-    ret = _pyi_pyconfig_set_module_search_paths(config, 3, module_search_paths_w);
+    ret = _pyi_pyconfig_set_module_search_paths(
+        config,
+        pyi_ctx->archive->python_version,
+        3,
+        module_search_paths_w
+    );
 
 end:
     /* Cleanup */
@@ -487,7 +502,7 @@ end:
  * Set program arguments (sys.argv).
  */
 static int
-_pyi_pyconfig_set_argv(PyConfig *config, int argc, wchar_t **argv_w)
+_pyi_pyconfig_set_argv(PyConfig *config, int python_version, int argc, wchar_t **argv_w)
 {
     /* Macro to avoid manual code repetition. */
     #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
@@ -499,7 +514,7 @@ _pyi_pyconfig_set_argv(PyConfig *config, int argc, wchar_t **argv_w)
     }
     /* Macro end */
 
-    switch (pyvers) {
+    switch (python_version) {
         _IMPL_CASE(308, PyConfig_v38)
         _IMPL_CASE(309, PyConfig_v39)
         _IMPL_CASE(310, PyConfig_v310)
@@ -516,26 +531,58 @@ _pyi_pyconfig_set_argv(PyConfig *config, int argc, wchar_t **argv_w)
 }
 
 
+#ifdef _WIN32
+
+/* On Windows, the command-line arguments are already available in array
+ * of wide-char strings, we can directly pass it into Python's
+ * configuration structure. */
 int
-pyi_pyconfig_set_argv(PyConfig *config, const ARCHIVE_STATUS *archive_status)
+pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
 {
+    return _pyi_pyconfig_set_argv(
+        config,
+        pyi_ctx->archive->python_version,
+        pyi_ctx->argc,
+        pyi_ctx->argv_w
+    );
+}
+
+#else /* ifdef _WIN32 */
+
+/* On POSIX systems, we need to convert command-line arguments from
+ * their local 8-bit encoding into wide-char strings used by Python
+ * configuration structure. This is done using `Py_DecodeLocale`
+ * function, which accounts for the locale/encoding that was set up
+ * during pre-initialization of Python interpreter. */
+int
+pyi_pyconfig_set_argv(PyConfig *config, const PYI_CONTEXT *pyi_ctx)
+{
+    char *const *argv;
     wchar_t **argv_w;
+    int argc;
     int ret = 0;
     int i;
 
+    /* Select original argc/argv vs. modified pyi_argc/pyi_argv */
+    if (pyi_ctx->pyi_argv != NULL) {
+        /* Modified pyi_argc/pyi_argv are available; use those */
+        argc = pyi_ctx->pyi_argc;
+        argv = pyi_ctx->pyi_argv;
+    } else {
+        /* Use original argc/argv */
+        argc = pyi_ctx->argc;
+        argv = pyi_ctx->argv;
+    }
+
     /* Allocate */
-    argv_w = calloc(archive_status->argc, sizeof(wchar_t *));
+    argv_w = calloc(argc, sizeof(wchar_t *));
     if (argv_w == NULL) {
         return -1;
     }
 
     /* Convert */
-    for (i = 0; i < archive_status->argc; i++) {
-#ifdef _WIN32
-        argv_w[i] = pyi_win32_utils_from_utf8(NULL, archive_status->argv[i], 0);
-#else
-        argv_w[i] = PI_Py_DecodeLocale(archive_status->argv[i], NULL);
-#endif
+    for (i = 0; i < argc; i++) {
+        argv_w[i] = PI_Py_DecodeLocale(argv[i], NULL);
         if (argv_w[i] == NULL) {
             /* Do not break; we need to initialize all elements */
             ret = -1;
@@ -546,28 +593,31 @@ pyi_pyconfig_set_argv(PyConfig *config, const ARCHIVE_STATUS *archive_status)
     }
 
     /* Set */
-    ret = _pyi_pyconfig_set_argv(config, archive_status->argc, argv_w);
+    ret = _pyi_pyconfig_set_argv(
+        config,
+        pyi_ctx->archive->python_version,
+        argc,
+        argv_w
+    );
 
 end:
     /* Cleanup */
-    for (i = 0; i < archive_status->argc; i++) {
-#ifdef _WIN32
-        free(argv_w[i]);
-#else
+    for (i = 0; i < argc; i++) {
         PI_PyMem_RawFree(argv_w[i]);
-#endif
     }
     free(argv_w);
 
     return ret;
 }
 
+#endif /* ifdef _WIN32 */
+
 
 /*
  * Set run-time options.
  */
 int
-pyi_pyconfig_set_runtime_options(PyConfig *config, const PyiRuntimeOptions *runtime_options)
+pyi_pyconfig_set_runtime_options(PyConfig *config, int python_version, const PyiRuntimeOptions *runtime_options)
 {
     /* Macro to avoid manual code repetition. */
     #define _IMPL_CASE(PY_VERSION, PYCONFIG_IMPL) \
@@ -616,7 +666,7 @@ pyi_pyconfig_set_runtime_options(PyConfig *config, const PyiRuntimeOptions *runt
     }
     /* Macro end */
 
-    switch (pyvers) {
+    switch (python_version) {
         _IMPL_CASE(308, PyConfig_v38)
         _IMPL_CASE(309, PyConfig_v39)
         _IMPL_CASE(310, PyConfig_v310)
@@ -642,9 +692,13 @@ pyi_pyconfig_preinit_python(const PyiRuntimeOptions *runtime_options)
     PyPreConfig_Common config;
 
     PI_PyPreConfig_InitIsolatedConfig((PyPreConfig *)&config);
+
     config.utf8_mode = runtime_options->utf8_mode;
     config.dev_mode = runtime_options->dev_mode;
 
+    /* Set the LC_CTYPE locale to the user-preferred locale, so it can be read using `locale.getlocale()` in python code. */
+    config.configure_locale = 1;
+
     /* Pre-initialize */
     PyStatus status = PI_Py_PreInitialize((const PyPreConfig *)&config);
     return PI_PyStatus_Exception(status) ? -1 : 0;
